from urllib.parse import quote

from django.contrib.auth import get_user_model
from django.http import HttpResponseRedirect
from django.urls import reverse

from social_core.pipeline import social_auth, user as social_core_user
from social_core.pipeline.user import USER_FIELDS

from .config import UNICEF_EMAIL


def social_details(backend, details, response, *args, **kwargs):
    r = social_auth.social_details(backend, details, response, *args, **kwargs)
    user = kwargs.get('user', None)
    if user:
        # here we are preventing messing up between current us and social user
        unauthorized = reverse('unicef_security:unauthorized')
        return HttpResponseRedirect(f"{unauthorized}?eu={user.email}&msgc=alreadyauthenticated")

    r['details']['idp'] = response.get('idp')
    if not r['details'].get('email'):
        if not response.get('email'):
            r['details']['email'] = response["signInNames.emailAddress"]
        else:
            r['details']['email'] = response.get('email')
    email = r['details'].get('email')
    if isinstance(email, str):
        r['details']['email'] = email.lower().strip()
    return r


def get_username(strategy, details, backend, user=None, *args, **kwargs):
    username = details.get('email')

    try:
        get_user_model().objects.get(username=username)

    except get_user_model().DoesNotExist:
        email = quote(username)
        unauthorized = reverse('unicef_security:unauthorized')
        return HttpResponseRedirect(f"{unauthorized}?eu={email}&msgc=nouser")

    return {'username': details.get('email')}


def create_unicef_user(strategy, details, backend, user=None, *args, **kwargs):
    """Overrides create_user, to create only UNICEF users"""
    if user:
        return {'is_new': False}

    fields = dict((name, kwargs.get(name, details.get(name)))
                  for name in backend.setting('USER_FIELDS', USER_FIELDS))

    if not (fields and details.get('email', '').endswith(UNICEF_EMAIL)):
        return

    response = kwargs.get('response')
    if response:
        email = response.get('email') or response.get("signInNames.emailAddress")
        if not email.endswith(UNICEF_EMAIL):
            return

    return {
        'is_new': True,
        'user': strategy.create_user(**fields)
    }


def user_details(strategy, details, backend, user=None, *args, **kwargs):
    """ This is where we update the user see what the property to map by is here updates_available = False """

    if user:
        # user_groups = [group.name for group in user.groups.all()]
        # business_area_code = details.get("business_area_code", 'defaultBA1235')
        # Update username with email and unusable password
        user.username = user.email
        user.first_name = details['first_name']
        user.last_name = details['last_name']
        user.set_unusable_password()
        user.save()

    return social_core_user.user_details(strategy, details, backend, user, *args, **kwargs)



import vcr

VCR = vcr.VCR(serializer='yaml',
              record_mode='once',
              match_on=['uri', 'method'],
              filter_headers=['authorization', 'token'],
              filter_post_data_parameters=['client_id', 'client_secret'])



import os

from django.conf import settings

AZURE_CLIENT_ID = os.environ.get('AZURE_CLIENT_ID', '')
AZURE_CLIENT_SECRET = os.environ.get('AZURE_CLIENT_SECRET', '')

GRAPH_CLIENT_ID = os.environ.get('GRAPH_CLIENT_ID', AZURE_CLIENT_ID)
GRAPH_CLIENT_SECRET = os.environ.get('GRAPH_CLIENT_SECRET', AZURE_CLIENT_SECRET)

AZURE_SSL = True
AZURE_URL_EXPIRATION_SECS = 10800
AZURE_ACCESS_POLICY_EXPIRY = 10800  # length of time before signature expires in seconds
AZURE_ACCESS_POLICY_PERMISSION = 'r'
AZURE_TOKEN_URL = 'https://login.microsoftonline.com/unicef.org/oauth2/token'
AZURE_GRAPH_API_BASE_URL = 'https://graph.microsoft.com'
AZURE_GRAPH_API_VERSION = 'v1.0'
AZURE_GRAPH_API_PAGE_SIZE = 300

AZURE_LOGOUT_BASE_URL = getattr(settings, 'SOCIAL_AUTH_TENANT_B2C_URL', getattr(
    settings, 'TENANT_B2C_URL', os.environ.get('AZURE_LOGOUT_BASE_URL', 'login.microsoftonline.com')))

AZURE_POLICY = getattr(
    settings, 'SOCIAL_AUTH_POLICY', getattr(settings, 'TENANT_POLICY', ''))

AZURE_TENANT_ID = getattr(settings, 'SOCIAL_AUTH_TENANT_ID', getattr(settings, 'TENANT_ID', '-'))

UNICEF_EMAIL = os.environ.get('UNICEF_EMAIL', '@unicef.org')



from django.urls import re_path

from unicef_security.views import UnauthorizedView, UNICEFLogoutView

app_name = 'unicef_security'

urlpatterns = [
    re_path(r'^unicef-logout/', UNICEFLogoutView.as_view(), name='unicef-logout'),
    re_path(r'^unauthorized/$', UnauthorizedView.as_view(), name="unauthorized"),
]



# Generated by Django 3.2 on 2022-04-06 23:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('unicef_security', '0002_auto_20201028_0154'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='user',
            options={'verbose_name': 'user', 'verbose_name_plural': 'users'},
        ),
        migrations.DeleteModel(
            name='BusinessArea',
        ),
        migrations.DeleteModel(
            name='Region',
        ),
    ]



import logging

from django import forms
from django.contrib import admin, messages
from django.contrib.admin import SimpleListFilter
from django.contrib.auth.admin import UserAdmin
from django.template.response import TemplateResponse
from django.utils.translation import gettext_lazy as _

from admin_extra_buttons.decorators import button
from admin_extra_buttons.mixins import ExtraButtonsMixin

from .config import UNICEF_EMAIL
from .graph import default_group, Synchronizer, SyncResult
from .models import User

logger = logging.getLogger(__name__)


class LoadUsersForm(forms.Form):
    emails = forms.CharField(widget=forms.Textarea)


class UNICEFUserFilter(SimpleListFilter):
    title = 'UNICEF user filter'
    parameter_name = 'email'

    def lookups(self, request, model_admin):
        return [
            ('unicef', 'UNICEF'),
            ('external', 'External'),
        ]

    def queryset(self, request, queryset):
        if self.value() == 'unicef':
            return queryset.filter(email__endswith=UNICEF_EMAIL)
        elif self.value() == 'external':
            return queryset.exclude(email__endswith=UNICEF_EMAIL)
        return queryset


@admin.register(User)
class UserAdminPlus(ExtraButtonsMixin, UserAdmin):
    list_display = ['username', 'display_name', 'email', 'is_staff',
                    'is_active', 'is_superuser', 'is_linked', 'last_login']
    list_filter = ['is_superuser', 'is_staff', 'is_active', UNICEFUserFilter]
    search_fields = ['username', 'display_name']
    fieldsets = (
        (None, {'fields': (('username', 'azure_id'), 'password')}),
        (_('Personal info'), {'fields': (('first_name', 'last_name',),
                                         ('email', 'display_name'),
                                         ('job_title',),
                                         )}),
        (_('Permissions'), {'fields': ('is_active', 'is_staff', 'is_superuser',
                                       'groups', 'user_permissions')}),
        (_('Important dates'), {'fields': ('last_login', 'date_joined')}),
    )
    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('username', 'password1', 'password2'),
        }),
    )
    readonly_fields = ('azure_id', 'job_title', 'display_name')

    def is_linked(self, obj):
        return bool(obj.azure_id)

    is_linked.boolean = True

    @button(label='Sync')
    def sync_user(self, request, pk):
        obj = self.get_object(request, pk)
        try:
            syncronizer = Synchronizer()
            syncronizer.sync_user(obj)
        except BaseException as e:
            self.message_user(request, str(e), messages.ERROR)

        self.message_user(request, "User synchronized")

    @button(label='Link user')
    def link_user_data(self, request, pk):
        opts = self.model._meta
        ctx = {
            'opts': opts,
            'app_label': 'security',
            'change': True,
            'is_popup': False,
            'save_as': False,
            'has_delete_permission': False,
            'has_add_permission': False,
            'has_change_permission': True,
        }
        obj = self.get_object(request, pk)
        syncronizer = Synchronizer()
        try:
            if request.method == 'POST':
                if request.POST.get('selection'):
                    data = syncronizer.get_user(request.POST.get('selection'))
                    syncronizer.sync_user(obj, data['id'])
                    self.message_user(request, "User linked")
                    return None
                else:
                    ctx['message'] = 'Select one entry to link'

            data = syncronizer.search_users(obj)
            ctx['data'] = data
            return TemplateResponse(request, 'admin/link_user.html', ctx)

        except BaseException as e:
            self.message_user(request, str(e), messages.ERROR)

    @button()
    def load(self, request):
        opts = self.model._meta
        ctx = {
            'opts': opts,
            'app_label': 'security',
            'change': True,
            'is_popup': False,
            'save_as': False,
            'has_delete_permission': False,
            'has_add_permission': False,
            'has_change_permission': True,
        }
        if request.method == 'POST':
            form = LoadUsersForm(request.POST)
            if form.is_valid():
                synchronizer = Synchronizer()
                emails = form.cleaned_data['emails'].split()
                total_results = SyncResult()
                for email in emails:
                    result = synchronizer.fetch_users("startswith(mail,'%s')" % email,
                                                      callback=default_group)
                    total_results += result
                self.message_user(request,
                                  f"{len(total_results.created)} users have been created, "
                                  f"{len(total_results.updated)} updated. "
                                  f"{len(total_results.skipped)} invalid entries found.")
        else:
            form = LoadUsersForm()
        ctx['form'] = form
        return TemplateResponse(request, 'admin/load_users.html', ctx)



import os
from pathlib import Path

import pytest

from tests.fixtures import VCR
from unicef_security.graph import Synchronizer


@VCR.use_cassette(str(Path(__file__).parent / 'vcr_cassettes/test_token.yml'))
@pytest.mark.skipif(os.environ.get("CIRCLECI") == "true", reason="Skip in CirlceCI")
def test_token():
    s = Synchronizer()
    assert s.access_token


@pytest.mark.skipif(os.environ.get("CIRCLECI") == "true", reason="Skip in CirlceCI")
@VCR.use_cassette(str(Path(__file__).parent / 'vcr_cassettes/test_user_data.yml'))
def test_user_data():
    s = Synchronizer()
    info = s.get_user('sapostolico@unicef.org')
    assert info['displayName']
    assert info['givenName']
    assert info['id']
    assert sorted(info.keys()) == sorted(['@odata.context', 'id', 'businessPhones',
                                          'displayName', 'givenName', 'jobTitle',
                                          'mail', 'mobilePhone', 'officeLocation',
                                          'preferredLanguage', 'surname',
                                          'userPrincipalName'])



import logging

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group

import requests
from constance import config as constance
from social_django.models import UserSocialAuth

from unicef_security.config import GRAPH_CLIENT_ID, GRAPH_CLIENT_SECRET

from . import config

AZURE_GRAPH_API_TOKEN_CACHE_KEY = 'azure_graph_api_token_cache_key'
AZURE_GRAPH_DELTA_LINK_KEY = 'azure_graph_delta_link_key'

logger = logging.getLogger(__name__)

DJANGOUSERMAP = {'_pk': ['username'],
                 'username': 'userPrincipalName',
                 'email': 'mail',
                 'azure_id': 'id',
                 'job_title': 'jobTitle',
                 'display_name': 'displayName',
                 'first_name': 'givenName',
                 'last_name': 'surname'}

ADMIN_EMAILS = [i[1] for i in settings.ADMINS]


def default_group(**kwargs):
    is_new = kwargs.get('is_new', False)
    user = kwargs.get('user', None)
    if is_new:
        if user.email in ADMIN_EMAILS:
            user.is_staff = True
            user.is_superuser = True
            user.save()
        else:
            g = Group.objects.filter(name=constance.DEFAULT_GROUP).first()
            if g:
                user.groups.add(g)


def get_unicef_user(backend, details, response, *args, **kwargs):
    User = get_user_model()
    if details.get('email'):
        filters = {'email': details['email']}
    elif details.get('unique_name'):
        filters = {'username': details['unique_name']}
    elif details.get('username'):
        filters = {'username': details['username']}

    try:
        user = User.objects.get(**filters)
        social = user.social_auth.get()
        user.social_user = social
        created = False
    except (User.DoesNotExist, UserSocialAuth.DoesNotExist):
        for k, v in response.items():
            if k in ['email', 'family_name', 'unique_name']:
                details[k] = v
        try:
            sync = Synchronizer()
            data = sync.get_user(details['email'])

            for k, v in data.items():
                details[k] = v

        except BaseException as e:
            logger.error(e)

        user, created = User.objects.get_or_create(
            username=details['unique_name'],
            defaults={'first_name': details.get('givenName', ''),
                      'last_name': details.get('surname', ''),
                      'job_title': details.get('jobTitle', ''),
                      'display_name': details.get('displayName', details['unique_name']),
                      'email': details.get('email', ''),
                      'azure_id': details.get('id'),
                      }
        )
        social, __ = UserSocialAuth.objects.get_or_create(user=user,
                                                          provider=backend.name,
                                                          uid=user.username
                                                          )
        user.social_user = social
    return {'user': user,
            'social': social,
            'uid': details.get('id'),
            'is_new': created}


class SyncResult:
    def __init__(self, keep_records=False):
        """
        :param keep_records: if True keep track of record istances
        """
        self.created = []
        self.updated = []
        self.skipped = []
        self.keep_records = keep_records

    def log(self, *result):
        if len(result) == 2:
            obj = {True: result[0],
                   False: result[0].pk}[self.keep_records]
            if result[1]:
                self.created.append(obj)
            else:
                self.updated.append(obj)
        else:
            self.skipped.append(result)

    def __add__(self, other):
        if isinstance(other, SyncResult):
            ret = SyncResult()
            ret.created.extend(other.created)
            ret.updated.extend(other.updated)
            ret.skipped.extend(other.skipped)
            return ret
        else:
            raise ValueError("Cannot add %s to SyncResult object" % type(other))

    def __repr__(self):
        return f"<SyncResult: {len(self.created)} {len(self.updated)} {len(self.skipped)}>"

    def __eq__(self, other):
        if isinstance(other, SyncResult):
            return self.created == other.created and self.updated == other.updated and self.skipped == other.skipped
        return False


NotSet = object()


class Synchronizer:
    def __init__(self, user_model=None, mapping=None, echo=None, id=None, secret=None):
        self.id = id or GRAPH_CLIENT_ID
        self.secret = secret or GRAPH_CLIENT_SECRET

        self.user_model = user_model or get_user_model()
        self.field_map = dict(mapping or DJANGOUSERMAP)
        self.user_pk_fields = self.field_map.pop('_pk')
        self._baseurl = '{}/{}/users'.format(config.AZURE_GRAPH_API_BASE_URL,
                                             config.AZURE_GRAPH_API_VERSION)
        self.startUrl = "%s/delta" % self._baseurl
        self.access_token = self.get_token()
        self.next_link = None
        self._delta_link = ''
        self.echo = echo or (lambda l: True)

    def get_token(self):
        if not self.id and self.secret:
            raise ValueError("Configure AZURE_CLIENT_ID and/or AZURE_CLIENT_SECRET")
        post_dict = {'grant_type': 'client_credentials',
                     'client_id': self.id,
                     'client_secret': self.secret,
                     'resource': config.AZURE_GRAPH_API_BASE_URL}
        response = requests.post(config.AZURE_TOKEN_URL, post_dict)
        if response.status_code != 200:  # pragma: no cover
            logger.error(f"Unable to fetch token from Azure. {response.status_code} {response.content}")
            raise BaseException(f'Error during token retrieval: {response.status_code} {response.content}')
        jresponse = response.json()
        token = jresponse['access_token']
        return token

    @property
    def delta_link(self):
        return self._delta_link

    @delta_link.setter
    def delta_link(self, value):
        self._delta_link = value

    def get_page(self, url, single=False):
        while True:
            headers = {'Authorization': 'Bearer {}'.format(self.get_token())}
            try:
                response = requests.get(url, headers=headers)
                if response.status_code == 401:
                    data = response.json()
                    if data["error"]["message"] == "Access token has expired.":
                        continue
                    else:
                        raise ConnectionError(f'400: Error processing the response {response.content}')

                elif response.status_code != 200:
                    raise ConnectionError(f'Code {response.status_code}. '
                                          f'Error processing the response {response.content}')
                break
            except ConnectionError as e:
                logger.exception(e)
                raise

        jresponse = response.json()
        self.next_link = jresponse.get('@odata.nextLink', None)
        self.delta_link = jresponse.get('@odata.deltaLink', None)
        if single:
            return jresponse
        return jresponse.get('value', [])

    def __iter__(self):
        values = self.get_page(self.startUrl)
        pages = 1
        while True:
            try:
                yield values.pop()
            except IndexError:
                if not self.next_link:
                    logger.debug(f"All pages  fetched. deltaLink: {self.delta_link}")
                    break
                values = self.get_page(self.next_link)
                logger.debug(f"fetched page {pages}")
                pages += 1
            except KeyboardInterrupt:
                break
            except BaseException as e:
                logger.exception(e)
                break

    def get_record(self, user_info: dict) -> (dict, dict):
        data = {fieldname: user_info.get(mapped_name, '')
                for fieldname, mapped_name in self.field_map.items()}
        pk = {fieldname: data.pop(fieldname) for fieldname in self.user_pk_fields}
        return pk, data

    def fetch_users(self, filter, callback=None):
        self.startUrl = "%s?$filter=%s" % (self._baseurl, filter)
        return self.syncronize(callback=callback)

    def search_users(self, record):
        url = "%s?$filter=" % self._baseurl
        filters = []
        if record.email:
            filters.append("mail eq '%s'" % record.email)
        if record.last_name:
            filters.append("surname eq '%s'" % record.last_name)
        if record.first_name:
            filters.append("givenName eq '%s'" % record.first_name)

        page = self.get_page(url + " or ".join(filters), single=True)
        return page['value']

    def filter_users_by_email(self, email):
        """https://graph.microsoft.com/v1.0/users?$filter=mail eq 'sapostolico@unicef.org'"""
        url = "%s?$filter=mail eq '%s'" % (self._baseurl, email)
        page = self.get_page(url, single=True)
        return page['value']

    def get_user(self, username):
        url = "%s/%s" % (self._baseurl, username)
        user_info = self.get_page(url, single=True)
        return user_info

    def sync_user(self, user, azure_id=None):
        if not (azure_id or user.azure_id):
            raise ValueError("Cannot sync user without azure_id")
        url = "%s/%s" % (self._baseurl, azure_id or user.azure_id)
        user_info = self.get_page(url, single=True)
        pk, values = self.get_record(user_info)
        user, __ = self.user_model.objects.update_or_create(**pk,
                                                            defaults=values)
        return user

    def resume(self, *, delta_link=None, max_records=None):
        if delta_link:
            self.startUrl = delta_link
        return self.syncronize(max_records)

    def is_valid(self, user_info):
        return user_info.get('email')

    def syncronize(self, max_records=None, callback=None):
        logger.debug("Start Azure user synchronization")
        results = SyncResult()
        try:
            for i, user_info in enumerate(iter(self)):
                pk, values = self.get_record(user_info)
                if self.is_valid(values):
                    user, created = self.user_model.objects.update_or_create(**pk,
                                                                             defaults=values)
                    if callback:
                        callback(user=user, is_new=created)
                    self.echo([user, created])
                    results.log(user, created)
                else:
                    results.log(user_info)
                if max_records and i > max_records:
                    break
        except BaseException as e:
            logger.exception(e)
            raise
        logger.debug(f"End Azure user synchronization: {results}")
        return results



import os

from django.conf import settings
from django.urls import reverse

from jwt import decode as jwt_decode, DecodeError, ExpiredSignatureError, get_unverified_header
from social_core.backends.azuread_b2c import AzureADB2COAuth2
from social_core.backends.azuread_tenant import AzureADTenantOAuth2
from social_core.exceptions import AuthTokenError


class UNICEFAzureADTenantOAuth2Ext(AzureADTenantOAuth2):
    name = 'unicef-azuread-tenant-oauth2'

    def user_data(self, access_token, *args, **kwargs):
        response = kwargs.get('response')
        id_token = response.get('id_token')

        # get key id and algorithm
        key_id = get_unverified_header(id_token)['kid']
        key = ''
        verify = os.environ.get('OAUTH2_VERIFY', False)
        try:
            # retrieve certificate for key_id
            if verify:
                certificate = self.get_certificate(key_id)
                key = certificate.public_key()

            options = {'verify_signature': verify}
            return jwt_decode(
                id_token,
                key=key,
                algorithms=['RS256'],
                audience=self.setting('KEY'),
                options=options,
            )
        except (DecodeError, ExpiredSignatureError) as error:
            raise AuthTokenError(self, error)


class UNICEFAzureADB2COAuth2(AzureADB2COAuth2):
    """UNICEF Azure ADB2C Custom Backend"""

    name = 'unicef-azuread-b2c-oauth2'
    BASE_URL = 'https://{tenant_name}.b2clogin.com/{tenant_id}'

    @property
    def base_url(self):
        return self.BASE_URL.format(tenant_name=self.tenant_name, tenant_id=self.tenant_id)

    @property
    def tenant_name(self):
        return self.setting('TENANT_NAME', 'unicef')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.redirect_uri = settings.HOST + reverse('social:complete', kwargs={'backend': 'unicef-azuread-b2c-oauth2'})



from social_core.tests.backends.test_azuread_b2c import AzureADOAuth2Test


class UNICEFAzureADTenantOAuth2ExtTest(AzureADOAuth2Test):
    backend_path = 'unicef_security.backends.UNICEFAzureADTenantOAuth2Ext'



from django.conf import settings
from django.views.generic import RedirectView, TemplateView

from unicef_security import config
from unicef_security.config import UNICEF_EMAIL


class UnauthorizedView(TemplateView):
    template_name = 'unauthorized.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['unicef_user'] = self.request.user.is_authenticated and self.request.user.email.endswith(UNICEF_EMAIL)
        return context


class UNICEFLogoutView(RedirectView):
    def get_redirect_url(self, *args, **kwargs):

        base_url = config.AZURE_LOGOUT_BASE_URL
        tenant_id = config.AZURE_TENANT_ID
        policy = config.AZURE_POLICY

        return f'https://{base_url}/{tenant_id}/{policy}/oauth2/v2.0/' \
               f'logout?post_logout_redirect_uri={settings.HOST}{settings.LOGOUT_URL}'



import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '&1_8z#^^_nz1o4xuvz4wqq&m&gpe5uq_(=!%sk=cg5e&o^ljqu'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'unicef_security',
    'admin_extra_buttons',
    'constance',
    'constance.backends.database',
    'social_django',
    'demo',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

AUTHENTICATION_BACKENDS = (
    'unicef_security.backends.UNICEFAzureADB2COAuth2',
    'django.contrib.auth.backends.ModelBackend',
)

ROOT_URLCONF = 'demo.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'demo.wsgi.application'


# Database
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


# Internationalization
LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
USE_I18N = True
USE_TZ = True

HOST = 'http://localhost'
# Static files (CSS, JavaScript, Images)
STATIC_URL = '/static/'

MEDIA_ROOT = '/tmp/'
VISION_LOGGER_MODEL = 'vision.VisionLog'

AUTH_USER_MODEL = 'demo.User'

CONSTANCE_CONFIG = {
    'DEFAULT_GROUP': ('', 'test_grp'),
}

CONSTANCE_BACKEND = 'constance.backends.database.DatabaseBackend'



from social_django.utils import load_backend, load_strategy

import pytest

from unicef_security.pipeline import create_unicef_user, get_username, social_details


@pytest.fixture()
def strategy():
    return load_strategy()


@pytest.fixture()
def backend(strategy):
    return load_backend(strategy=strategy, name='unicef-azuread-b2c-oauth2', redirect_uri='/')


@pytest.fixture()
def response(auth_user):
    return {
        'id_token': 'token',
        'token_type': 'Bearer',
        'not_before': 1607541825,
        'id_token_expires_in': 3600,
        'profile_info': 'profile_info',
        'scope': 'openid',
        'access_token': 'access_token',
        'exp': 1607545425,
        'nbf': 1607541825,
        'ver': '1.0',
        'iss': 'https://tenant.b2clogin.com/1234567890/v2.0/',
        'sub': 'abcdefgh', 'aud': 'abcdefgh',
        'acr': 'b2c_1a_unicef_social_signup_signin',
        'iat': 1607541825,
        'auth_time': 1607541824,
        'given_name': 'Given',
        'family_name': 'Family',
        'name': 'Given Family',
        'idp': 'UNICEF Azure AD',
        'email': auth_user.email
    }


@pytest.fixture()
def details(auth_user):
    return {
        'username': 'Given Family',
        'email': auth_user.email,
        'fullname': 'Given Family',
        'first_name': 'Given',
        'last_name': 'Family',
        'idp': 'UNICEF Azure AD'
    }


@pytest.mark.django_db
def test_social_details(auth_user, backend, details, response):
    return_dict = social_details(backend, details, response)
    assert 'idp' in return_dict['details']
    assert return_dict['details']['email'] == auth_user.email


@pytest.mark.django_db
def test_get_username(strategy, backend, details, auth_user):
    result_dict = get_username(strategy, details, backend, user=auth_user)
    assert 'username' in result_dict
    assert result_dict['username'] == auth_user.email


@pytest.mark.django_db
def test_create_unicef_user_ok(strategy, details, backend):
    result_dict = create_unicef_user(strategy, details, backend)
    assert result_dict['is_new']
    assert result_dict['user']


@pytest.mark.django_db
def test_create_unicef_user_ko(strategy, details, backend):
    details['email'] = 'test_undeliverable@example.com'
    result_dict = create_unicef_user(strategy, details, backend)
    assert not result_dict


@pytest.mark.django_db
def test_create_unicef_user_existing(strategy, details, backend):
    result_dict = create_unicef_user(strategy, details, backend, 1)
    assert 'is_new' in result_dict
    assert not result_dict['is_new']



from django.contrib import admin
from django.urls import include, re_path

urlpatterns = [
    re_path(r'^admin/', admin.site.urls),
    re_path(r'social', include('social_django.urls', namespace='social')),
]



from demo.models import User


def test_user():
    user = User(username="user_name")
    assert str(user) == 'user_name'
    assert user.label == 'user_name'

    user = User(display_name="display_name")
    assert user.label == 'display_name'

    user = User(first_name="first_name")
    assert user.label == 'first_name'

    user = User(first_name="first_name", last_name="last_name")
    assert user.label == 'first_name last_name:'
